package ru.ldeloff.servermonitorbot.service.command.uname;

import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.telegram.telegrambots.meta.api.methods.send.SendMessage;
import ru.ldeloff.servermonitorbot.model.Command;
import ru.ldeloff.servermonitorbot.model.SshServer;
import ru.ldeloff.servermonitorbot.model.User;
import ru.ldeloff.servermonitorbot.service.command.CommandTemplate;
import ru.ldeloff.servermonitorbot.service.role.RoleService;
import ru.ldeloff.servermonitorbot.service.ssh.SshService;
import ru.ldeloff.servermonitorbot.service.user.UserService;
import ru.ldeloff.servermonitorbot.utils.SshServerUtils;

import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.Optional;

@Service
@Slf4j
public class UnameCommand extends CommandTemplate {
    final SshService sshService;
    public UnameCommand(UserService userService, RoleService roleService, SshService sshService) {
        super(userService, roleService);
        this.sshService = sshService;
    }
    @Override
    public SendMessage actionForAuth(User user, SendMessage message) {
        String [] tags = Arrays.stream(message.getText().split(":"))
                .map(String::trim)
                .toArray(String[]::new);

        if (tags[1].equals("all")) {
            message.setText(unameAllHost());
        } else {
            message.setText(Objects.requireNonNull(unameSpecificHost(tags[1])));
        }
        return message;
    }

    private String unameAllHost() {
        List<SshServer> servers = SshServerUtils.filterGoodServers(sshService.getSshServers());
        StringBuilder response = new StringBuilder();
        servers.forEach(server -> response.append(unameSpecificHost(server.getName())).append("\n"));
        return response.toString();
    }

    private String unameSpecificHost(String serverName) {
        Optional<SshServer> server = sshService.getSshServers()
                .stream()
                .filter(x -> x.getName().equals(serverName))
                .findFirst();
        if (server.isPresent()) {
            Command result = sshService.execute(new Command("uname -a", 100, server.get()));
            return server.get().getName() + ": " +
                    (Objects.isNull(result.getResponse()) ? "ошибка при выполнении команды" : result.getResponse());
        } else {
            log.error("Ошибка при выполнении команды 'uname'. Искомый сервер (" + serverName + ") не найден");
            return null;
        }
    }
}
